@@ -1,28 +1,45 @@
-#' Create a network from a collection of line strings.
+#' Create a network from a collection of line strings
 #'
-#' @param edges A data frame with the network edges
+#' @param edges An [`sf::sf`] or [`sf::sfc_LINESTRING`] object with the
+#'   network edges
 #' @param flatten Whether all intersections between edges should be
 #'   converted to nodes
 #' @param clean Whether general cleaning tasks should be run on the generated
 #'   network (see [`clean_network()`] for the description of tasks)
 #'
-#' @return A spatial network object
+#' @return An [`sfnetworks::sfnetwork`] object
 #' @export
 #' @examples
 #' edges <- sf::st_sfc(
 #'   sf::st_linestring(matrix(c(0, 0, 1, 1), ncol = 2, byrow = TRUE)),
-#'   sf::st_linestring(matrix(c(0, 1, 1, 0), ncol = 2, byrow = TRUE))
+#'   sf::st_linestring(matrix(c(0, 1, 1, 0), ncol = 2, byrow = TRUE)),
+#'   crs = sf::st_crs("EPSG:32635")
 #' )
-#' sf::st_crs(edges) <- sf::st_crs("EPSG:4326")
+#'
+#' # Run with default values
 #' as_network(edges)
+#'
+#' # Only build the spatial network
+#' as_network(edges, flatten = FALSE, clean = FALSE)
+#' @srrstats {G2.7} The `edges` parameter only accepts tabular input of class
+#'   `sf`. `sfnetwork` objects are `sf`-compatible and are commonly used
+#'   for spatial network analysis.
+#' @srrstats {SP4.0, SP4.0b, SP4.1, SP4.2} The return value is of class
+#'   [`sfnetworks::sfnetwork`], explicitly documented as such, and it maintains
+#'   the same units as the input.
 as_network <- function(edges, flatten = TRUE, clean = TRUE) {
+  # Check input
+  checkmate::assert_multi_class(edges, c("sf", "sfc"))
+  checkmate::assert_logical(flatten, len = 1)
+  checkmate::assert_logical(clean, len = 1)
+
   network <- sfnetworks::as_sfnetwork(edges, directed = FALSE)
   if (flatten) network <- flatten_network(network)
   if (clean) network <- clean_network(network)
   network
 }
 
-#' Flatten a network by adding points at apparent intersections.
+#' Flatten a network by adding points at apparent intersections
 #'
 #' All crossing edges are identified, and the points of intersections are
 #' injected within the edge geometries. Note that the injected points are
@@ -34,17 +51,28 @@ as_network <- function(edges, flatten = TRUE, clean = TRUE) {
 #' [`sfnetworks::st_network_blend()`], but in that case an external point is
 #' only injected to the closest edge.
 #'
-#' @param network A network object
+#' @param network A network object of class [`sfnetworks::sfnetwork`]
 #'
-#' @return A network object with additional points at intersections
+#' @return An [`sfnetworks::sfnetwork`] object with additional points at
+#'   intersections
 #' @export
-#' @examples
+#' @examplesIf interactive()
 #' bucharest_osm <- get_osm_example_data()
 #' edges <- dplyr::bind_rows(bucharest_osm$streets,
 #'                           bucharest_osm$railways)
 #' network <- sfnetworks::as_sfnetwork(edges, directed = FALSE)
 #' flatten_network(network)
+#' @srrstats {G2.7} The `network` object provided as input must be of class
+#'   `sfnetwork`. `sfnetwork` objects are `sf`-compatible and are commonly used
+#'   for spatial network analysis.
+#' @srrstats {SP4.0, SP4.0a, SP4.1, SP4.2} The return value is of class
+#'   [`sfnetworks::sfnetwork`], same as the input class, explicitly documented
+#'   as such, and it maintains the same units as the input.
 flatten_network <- function(network) {
+  # Check input
+  as_crs(network)
+  checkmate::assert_class(network, "sfnetwork")
+
   nodes <- sf::st_as_sf(network, "nodes")
   edges <- sf::st_as_sf(network, "edges")
 
@@ -66,7 +94,17 @@ flatten_network <- function(network) {
   network_new
 }
 
+#' Get edges that cross each other.
+#'
 #' @noRd
+#' @srrstats {G2.10} This function uses `sf::st_geometry()` to extract the
+#'   geometry column from the `sf` object `edges`. This is used when only
+#'   geometry information is needed from that point onwards and all other
+#'   attributes (i.e., columns) can be safely discarded. The object returned
+#'   by `sf::st_geometry()` is a simple feature geometry list column of class
+#'   `sfc`.
+#' @srrstats {SP4.0, SP4.0b, SP4.1} The return value is of class [`sf::sf`] and
+#'   it maintains the same units as the input.
 get_crossing_edges <- function(edges) {
   geometry <- sf::st_geometry(edges)
   crossings <- sf::st_crosses(geometry)
@@ -74,7 +112,11 @@ get_crossing_edges <- function(edges) {
   sf::st_sf(id = which(mask), geometry = geometry[mask])
 }
 
+#' Get intersection points between edges.
+#'
 #' @noRd
+#' @srrstats {SP4.0, SP4.0b, SP4.1} The return value is of class
+#'   [`sf::sfc_POINT`] and it maintains the same units as the input.
 get_intersection_points <- function(edges) {
   # make sure edges is an sf object, so st_intersection also returns origins
   intersections <- sf::st_intersection(sf::st_sf(edges))
@@ -84,8 +126,19 @@ get_intersection_points <- function(edges) {
   sfheaders::sf_cast(points, to = "POINT")
 }
 
-#' @noRd
+#' Insert intersection points into edges.
+#'
 #' @importFrom utils head tail
+#'
+#' @noRd
+#' @srrstats {G2.10} This function uses `sf::st_geometry()` to extract the
+#'   geometry column from the `sf` object `edges`. This is used when only
+#'   geometry information is needed from that point onwards and all other
+#'   attributes (i.e., columns) can be safely discarded. The object returned
+#'   by `sf::st_geometry()` is a simple feature geometry list column of class
+#'   `sfc`.
+#' @srrstats {SP4.0, SP4.0a, SP4.1} The return value is of class
+#'   [`sf::sfc_LINESTRING`] and it maintains the same units as the input.
 insert_intersections <- function(edges, points, tol = 1.e-3) {
 
   edge_geometry <- sf::st_geometry(edges)
@@ -138,18 +191,25 @@ insert_intersections <- function(edges, points, tol = 1.e-3) {
   return(edges_new)
 }
 
+#' Check if a point is within a given edge.
+#'
 #' @noRd
 is_point_in_edge <- function(point, edge, tol) {
   any(calc_distance(point, edge) < tol)
 }
 
+#' Calculate the distance from a point to an edge.
+#'
 #' @noRd
 calc_distance <- function(point, edge) {
   sqrt((edge[, "x"] - point["X"]) ^ 2 + (edge[, "y"] - point["Y"]) ^ 2)
 }
 
-#' @noRd
+#' Calculate the rolling sum of a vector.
+#'
 #' @importFrom utils head tail
+#'
+#' @noRd
 calc_rolling_sum <- function(x, n = 2) {
   cs <- cumsum(x)
   # roll the cumsum array by adding `n` zeros at its beginning and dropping
@@ -160,7 +220,7 @@ calc_rolling_sum <- function(x, n = 2) {
   tail(cs - cs_roll, length(x) - 1)
 }
 
-#' Clean a spatial network.
+#' Clean a spatial network
 #'
 # nolint start
 #' Subdivide edges by [adding missing nodes](https://luukvdmeer.github.io/sfnetworks/articles/sfn02_preprocess_clean.html#subdivide-edges),
@@ -169,11 +229,11 @@ calc_rolling_sum <- function(x, n = 2) {
 #' and discard all but the main connected component.
 # nolint end
 #'
-#' @param network A network object
+#' @param network A network object of class [`sfnetworks::sfnetwork`]
 #' @param simplify Whether the network should be simplified with
 #'   [`simplify_network()`]
 #'
-#' @return A cleaned network object
+#' @return A cleaned network object of class [`sfnetworks::sfnetwork`]
 #' @export
 #' @examplesIf interactive()
 #' bucharest_osm <- get_osm_example_data()
@@ -181,10 +241,20 @@ calc_rolling_sum <- function(x, n = 2) {
 #'                           bucharest_osm$railways)
 #' network <- sfnetworks::as_sfnetwork(edges, directed = FALSE)
 #' clean_network(network)
+#' @srrstats {G2.7} The `network` object provided as input must be of class
+#'   `sfnetwork`. `sfnetwork` objects are `sf`-compatible and are commonly used
+#'   for spatial network analysis.
+#' @srrstats {SP4.0, SP4.0a, SP4.1, SP4.2} The return value is of class
+#'   [`sfnetworks::sfnetwork`], same as the input class, explicitly documented
+#'   as such, and it maintains the same units as the input.
 clean_network <- function(network, simplify = TRUE) {
+  # Check input
+  checkmate::assert_class(network, "sfnetwork")
+  checkmate::assert_logical(simplify, len = 1)
+
   # subdivide edges by adding missing nodes
   net <- tidygraph::convert(network, sfnetworks::to_spatial_subdivision,
-                            .clean = TRUE)
+                            .clean = TRUE) |> suppressWarnings()
   # run simplification steps
   if (simplify) net <- simplify_network(net)
   # remove pseudo-nodes
@@ -207,6 +277,9 @@ clean_network <- function(network, simplify = TRUE) {
 #'
 #' @return A simplifed network object
 #' @keywords internal
+#' @srrstats {SP4.0, SP4.0a, SP4.1, SP4.2} The return value is of class
+#'   [`sfnetworks::sfnetwork`], same as the input class, explicitly documented
+#'   as such, and it maintains the same units as the input.
 simplify_network <- function(network) {
   network |>
     sfnetworks::activate("edges") |>
@@ -245,9 +318,25 @@ simplify_network <- function(network) {
 #' @param weight_name Name of the column in the edge table where to add the
 #'   weights
 #'
-#' @return A network object with weights added as a column in the edge table
+#' @return A network object of class [`sfnetworks::sfnetwork`] with weights
+#'   added as a column in the edge table
 #' @importFrom rlang :=
 #' @keywords internal
+#'
+#' @srrstats {G2.4, G2.4b} Explicit conversion of logical vector to numeric with
+#'   `as.numeric()` used for calculating penalty weights.
+#' @srrstats {G2.7} The `network` object provided as input must be of class
+#'   `sfnetwork`. `sfnetwork` objects are `sf`-compatible and are commonly used
+#'   for spatial network analysis.
+#' @srrstats {G2.10} This function uses `sf::st_geometry()` to extract the
+#'   geometry column from the `sf` object `sf::st_as_sf(network, "edges")`.
+#'   This is used when only geometry information is needed from that point
+#'   onwards and all other attributes (i.e., columns) can be safely discarded.
+#'   The object returned by `sf::st_geometry()` is a simple feature geometry
+#'   list column of class `sfc`.
+#' @srrstats {SP4.0, SP4.0a, SP4.1, SP4.2} The return value is of class
+#'   [`sfnetworks::sfnetwork`], same as the input class, explicitly documented
+#'   as such, and it maintains the same units as the input.
 add_weights <- function(network, target = NULL, exclude_area = NULL,
                         penalty = 1000., weight_name = "weight") {
   edges <- sf::st_geometry(sf::st_as_sf(network, "edges"))
@@ -279,15 +368,24 @@ add_weights <- function(network, target = NULL, exclude_area = NULL,
 
 #' Find shortest path between a pair of nodes in the network.
 #'
-#' @param network A spatial network object
+#' @param network A spatial network object of class [`sfnetworks::sfnetwork`]
 #' @param from Start node
 #' @param to End node
 #' @param weights Name of the column in the network edge table from where to
 #'   take the weigths
 #'
-#' @return A simple feature geometry
+#' @return An [`sf::sfc_LINESTRING`] object
 #' @importFrom rlang .data
 #' @keywords internal
+#' @srrstats {G2.10} This function uses `sf::st_geometry()` to extract the
+#'   geometry column from the `sf` object `sf::st_as_sf(network, "edges")`.
+#'   This is used when only geometry information is needed from that point
+#'   onwards and all other attributes (i.e., columns) can be safely discarded.
+#'   The object returned by `sf::st_geometry()` is a simple feature geometry
+#'   list column of class `sfc`.
+#' @srrstats {SP4.0, SP4.0b, SP4.1, SP4.2} The return value is of class
+#'   [`sf::sfc_LINESTRING`], explicitly documented as such, and it maintains
+#'   the same units as the input.
 shortest_path <- function(network, from, to, weights = "weight") {
   paths <- sfnetworks::st_network_paths(
     network, from = from, to = to, weights = weights, type = "shortest",
@@ -309,11 +407,20 @@ shortest_path <- function(network, from, to, weights = "weight") {
 
 #' Find the node in a network that is closest to a target geometry.
 #'
-#' @param network A network object
-#' @param target The target geometry
+#' @param network A network object of class [`sfnetworks::sfnetwork`]
+#' @param target An object of class [`sf::sf`] or [`sf::sfc`]
 #'
-#' @return A node in the network as a simple feature geometry
+#' @return A node in the network as an object of class [`sf::sfc_POINT`]
 #' @keywords internal
+#' @srrstats {G2.10} This function uses `sf::st_geometry()` to extract the
+#'   geometry column from the `sf` object `sf::st_as_sf(network, "nodes")`.
+#'   This is used when only geometry information is needed from that point
+#'   onwards and all other attributes (i.e., columns) can be safely discarded.
+#'   The object returned by `sf::st_geometry()` is a simple feature geometry
+#'   list column of class `sfc`.
+#' @srrstats {SP4.0, SP4.0b, SP4.1, SP4.2} The return value is of class
+#'   [`sf::sfc_POINT`], explicitly documented as such, and it maintains
+#'   the same units as the input.
 nearest_node <- function(network, target) {
   nodes <- sf::st_as_sf(network, "nodes") |>
     sf::st_geometry()
@@ -326,14 +433,17 @@ nearest_node <- function(network, target) {
 #' If subsetting results in multiple disconnected components, we keep the main
 #' one.
 #'
-#' @param network A network object
+#' @param network A spatial network object of class [`sfnetworks::sfnetwork`]
 #' @param target The target geometry
 #' @param elements The elements of the network to filter. It can be "nodes"
 #'   or "edges"
 #'
-#' @return A spatial network object
+#' @return A spatial network object of class [`sfnetworks::sfnetwork`]
 #' @importFrom rlang !!
 #' @keywords internal
+#' @srrstats {SP4.0, SP4.0a, SP4.1, SP4.2} The return value is of class
+#'   [`sfnetworks::sfnetwork`], same as the input class, explicitly documented
+#'   as such, and it maintains the same units as the input.
 filter_network <- function(network, target, elements = "nodes") {
   if (elements == "nodes") {
     intersect_func <- sfnetworks::node_intersects
@@ -352,13 +462,17 @@ filter_network <- function(network, target, elements = "nodes") {
 
 #' Identify network edges that are intersecting a geometry
 #'
-#' @param network A spatial network object
-#' @param geometry A simple feature geometry
+#' @param network A spatial network object of class [`sfnetworks::sfnetwork`]
+#' @param geometry An object of class [`sf::sfc`]
 #' @param index Whether to return the indices of the matchin edges or the
 #'   geometries
 #'
 #' @return Indices or geometries of the edges intersecting the given geometry
+#'   of class [`sf::sfc_LINESTRING`]
 #' @keywords internal
+#' @srrstats {SP4.0, SP4.0b, SP4.1, SP4.2} If `index = FALSE`, the return value
+#'   is of class [`sf::sfc_LINESTRING`], explicitly documented as such, and it
+#'   maintains the same units as the input.
 get_intersecting_edges <- function(network, geometry, index = FALSE) {
   edges <- sf::st_as_sf(network, "edges")
   intersects <- sf::st_intersects(edges, geometry, sparse = FALSE)
@@ -372,8 +486,18 @@ get_intersecting_edges <- function(network, geometry, index = FALSE) {
 #' Find intersections between the edges of two networks
 #'
 #' @param network_1,network_2 The two spatial network objects
-#' @return A simple feature object
+#' @return An object of class [`sf::sfc`]
 #' @keywords internal
+#' @srrstats {G2.10} This function uses `sf::st_geometry()` to extract the
+#'   geometry column from the `sf` objects `sf::st_as_sf(network_1, "edges")`
+#'   and `sf::st_as_sf(network_2, "edges")`. This is used when only geometry
+#'   information is needed from that point onwards and all other attributes
+#'   (i.e., columns) can be safely discarded. The object returned by
+#'   `sf::st_geometry()` is a simple feature geometry list column of class
+#'   `sfc`.
+#' @srrstats {SP4.0, SP4.0b, SP4.1, SP4.2} The return value is of class
+#'   [`sf::sfc`], explicitly documented as such, and it maintains the same units
+#'   as the input.
 find_intersections <- function(network_1, network_2) {
   sf::st_intersection(sf::st_geometry(sf::st_as_sf(network_1, "edges")),
                       sf::st_geometry(sf::st_as_sf(network_2, "edges")))
